/*******************************************************************************
 * DISCLAIMER
 * This software is supplied by Renesas Electronics Corporation and is only
 * intended for use with Renesas products. No other uses are authorized. This
 * software is owned by Renesas Electronics Corporation and is protected under
 * all applicable laws, including copyright laws.
 * THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
 * THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
 * LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
 * AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
 * ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
 * FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
 * ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
 * BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this software
 * and to discontinue the availability of this software. By using this software,
 * you agree to the additional terms and conditions found by accessing the
 * following link:
 * http://www.renesas.com/disclaimer
 * Copyright 2021-2023 Renesas Electronics Corporation All rights reserved.
 *******************************************************************************/

/*******************************************************************************
 * DESCRIPTION   : remap driver header
 ******************************************************************************/
#ifndef REMAP_H_
#define REMAP_H_

#define ICUMX_CTLREG_BASE    (0xFFFEE200U)
#define ICUMX_CFREMAP        (ICUMX_CTLREG_BASE + 0x4CU)
#define CFREMAP_AREA_SIZE    (0x02000000U)

#include <remap_register.h>

typedef struct{
    uint32_t base_addr;    /* Base address of Region ID registers. */
    uint32_t rmp_addr;     /* Stores the address converted from the Region ID base address to SICREMAP address. */
} REMAP_TABLE;

uint32_t remap_get_phys_addr(uint32_t remap_addr);
uint32_t get_cfremap_addr(uint32_t fetch_addr);
uint32_t remap_get_remap_addr(uint32_t phys_addr);
void remap_register(uint32_t addr, uint32_t *remap_addr);
void remap_unregister(uint32_t remap_addr);
void set_sicremap_s4v10(void);
void set_sicremap_fcpr(void);
#endif /* REMAP_H_ */
